# app.py (using Flask)
from flask import Flask, request, jsonify
from bs4 import BeautifulSoup
import requests
import re
from urllib.parse import urlparse

app = Flask(__name__)

def count_keywords(text, keywords):
    text_lower = text.lower()
    exact_match_count = 0
    if keywords:
        # Regex for exact phrase match (case-insensitive)
        exact_match_count = len(re.findall(r'\b' + re.escape(" ".join(keywords)) + r'\b', text_lower))

    individual_keyword_counts = {}
    for kw in keywords:
        individual_keyword_counts[kw] = len(re.findall(r'\b' + re.escape(kw) + r'\b', text_lower))
    return exact_match_count, individual_keyword_counts

def get_text_content(soup, selector):
    elements = soup.select(selector)
    return " ".join([elem.get_text(separator=' ', strip=True) for elem in elements])

def get_meta_description(soup):
    meta_tag = soup.find('meta', attrs={'name': 'description'})
    return meta_tag['content'] if meta_tag else ''

def get_alt_texts(soup):
    img_tags = soup.find_all('img')
    return " ".join([img.get('alt', '') for img in img_tags])

def count_url_keywords(url_str, keywords):
    parsed_url = urlparse(url_str)
    path = parsed_url.path.lower()
    netloc = parsed_url.netloc.lower().replace("www.", "") # remove www.
    
    # Combine relevant parts of the URL for keyword checking
    # Split by common URL separators (/, -, .) to check for embedded words
    parts = re.split(r'[/\-.]', path + netloc)
    
    count = 0
    for keyword in keywords:
        for part in parts:
            if keyword.lower() in part: # Check if the keyword is a substring of any part
                count += 1
    return count

@app.route('/analyze-urls', methods=['POST'])
def analyze_urls():
    data = request.json
    your_url = data['yourUrl']
    keyword_input = data['keyword']
    compare_urls = data['compareUrls']

    all_urls = [your_url] + compare_urls
    analysis_results = {}

    keywords = keyword_input.lower().split(/\s+/).filter(w => w.length > 0) # Ensure consistency with JS parsing

    for url_str in all_urls:
        try:
            response = requests.get(url_str, timeout=10)
            response.raise_for_status() # Raise HTTPError for bad responses (4xx or 5xx)
            soup = BeautifulSoup(response.text, 'html.parser')

            title_text = soup.title.string if soup.title else ''
            meta_description_text = get_meta_description(soup)
            h1_text = get_text_content(soup, 'h1')
            h2_text = get_text_content(soup, 'h2')
            h3_h6_text = get_text_content(soup, 'h3, h4, h5, h6')
            body_text = get_text_content(soup, 'body')
            image_alt_text = get_alt_texts(soup)
            bold_strong_text = get_text_content(soup, 'b, strong')

            # Calculate word and character counts
            title_words = len(title_text.split())
            meta_desc_words = len(meta_description_text.split())
            h1_words = len(h1_text.split())
            h2_words = len(h2_text.split())
            h3_h6_words = len(h3_h6_text.split())
            body_words = len(body_text.split())
            img_alt_words = len(image_alt_text.split())
            bold_strong_words = len(bold_strong_text.split())

            title_chars = len(title_text)
            meta_desc_chars = len(meta_description_text)
            h1_chars = len(h1_text)
            h2_chars = len(h2_text)
            h3_h6_chars = len(h3_h6_text)
            body_chars = len(body_text)
            img_alt_chars = len(image_alt_text)
            bold_strong_chars = len(bold_strong_text)

            # Keyword counts
            elements_data = {
                'title': title_text,
                'meta description': meta_description_text,
                'h1': h1_text,
                'h2': h2_text,
                'h3-h6': h3_h6_text,
                'body': body_text,
                'image alt tags': image_alt_text,
                'bold and strong tags': bold_strong_text
            }

            url_keyword_count = count_url_keywords(url_str, keywords)

            url_analysis = {
                'url': {
                    'text': url_str,
                    'wordCount': len(url_str.split()), # Not very meaningful for URL, but included for consistency
                    'charCount': len(url_str),
                    'exactKeywordCount': count_keywords(url_str, keywords)[0], # Also not very meaningful for URL
                    'individualKeywordCounts': count_keywords(url_str, keywords)[1],
                    'density': '0.00%', # Density not usually applied to URL string itself
                    'urlKeywordCount': url_keyword_count
                }
            }

            for element_name, text_content in elements_data.items():
                exact_kw_count, individual_kw_counts = count_keywords(text_content, keywords)
                total_chars = len(text_content)
                density = (exact_kw_count * len(" ".join(keywords)) / total_chars * 100) if total_chars > 0 else 0.0
                
                url_analysis[element_name] = {
                    'text': text_content,
                    'wordCount': len(text_content.split()),
                    'charCount': total_chars,
                    'exactKeywordCount': exact_kw_count,
                    'individualKeywordCounts': individual_kw_counts,
                    'density': f"{density:.2f}%"
                }
            
            analysis_results[url_str] = url_analysis

        except requests.exceptions.RequestException as e:
            print(f"Error fetching {url_str}: {e}")
            analysis_results[url_str] = {"error": str(e), "message": "Could not fetch URL content."}
        except Exception as e:
            print(f"An unexpected error occurred for {url_str}: {e}")
            analysis_results[url_str] = {"error": str(e), "message": "An unexpected error occurred during processing."}

    return jsonify(analysis_results)

if __name__ == '__main__':
    app.run(debug=True) # Set debug=False in production